package models

import (
	"pro2d/common"
	"pro2d/common/components"
	"pro2d/csvdata"
	"pro2d/pb"
)

type HeroModel struct {
	components.ISchema
	Data       *pb.Hero
	Equipments common.IMapStringNum
}

func NewHero(hero *pb.Hero) *HeroModel {
	m := &HeroModel{
		ISchema:    NewSchema(hero.Id, hero),
		Data:       hero,
		Equipments: common.StringToMapNum(hero.Equipments),
	}
	return m
}

func (m *HeroModel) DelEquipment(key string) {
	delete(m.Equipments, key)
	m.SetProperty("equipments", common.MapNumToString(m.Equipments))
}

func (m *HeroModel) AddEquipment(key string, pos int32) {
	m.Equipments[key] = pos
	m.SetProperty("equipments", common.MapNumToString(m.Equipments))
}
func (m *HeroModel) UpLevel(exp int32) (int32, int32) {
	exp = exp + m.Data.Exp
	level := m.Data.Level
	oldLevelExp := csvdata.Get().TbHeroLevelExp.Get(level)
	if oldLevelExp == nil {
		return level, exp
	}

	for exp > 0 {
		if exp >= oldLevelExp.NeedExp {
			exp = exp - oldLevelExp.NeedExp
			level++
		} else {
			break
		}

		oldLevelExp = csvdata.Get().TbHeroLevelExp.Get(level)
		if oldLevelExp == nil {
			return level, exp
		}
	}

	return level, exp
}
